/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 */

#ifndef MISC_HPP_
#define MISC_HPP_

#include <vector>

using namespace std;

#define MAX_LINE 1024

typedef enum Optimization_method {opt_annealing, opt_bbm, opt_method_invalid} Optimization_method;
typedef enum Effort_type {easy, hard} Effort_type;
typedef enum ROUTING_TYPE {routing_xy, routing_oddeven} ROUTING_TYPE;
typedef enum Adaptive_routing_scheme {west_first, odd_even} Adaptive_routing_scheme;

typedef struct Param {

    // some program constants
    int var_max_length;
    int precision;

    // network system parameters
    int link_bandwidth;
    float buf_read_ebit;
    float buf_write_ebit;
    float link_ebit; 
    float switch_ebit;
    ROUTING_TYPE routing_type;
    Adaptive_routing_scheme legal_turn_set;
    Optimization_method optimization_method;
    Effort_type routing_effort;

    // mapping tool configruation
    int pq_size;   // priority queue size

    // mapping tool internal data structure
    int ***link_usage_matrix;
    vector<int> ** link_usage_list;

    int seed;

    // program control
    bool routing_table_synthesis;
    bool do_mapping;
    bool dump_mapping;
    bool read_mapping;
    bool dump_routing;
    bool dump_apcg;
    bool verbose;
    int min_hit_threshold;

    // file names
    char exec_str[MAX_LINE];
    char* dump_mapping_table_file;
    char* read_mapping_table_file;
    char* routing_table_file;
    char* dump_apcg_file;

    // option parsing related 
    bool parse_error;
    bool quit_flag;

    Param(void);
};
typedef Param *pParam; 


#endif // MISC_HPP_
