/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 */

#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include "tile.hpp"
#include "app.hpp"
#include "util.hpp"
#include "arch.hpp"
#include "mapping.hpp"
#include "misc.hpp"
#include "msg.hpp"

using namespace std;

Param param;

pLink gLink = NULL;
pTile gTile = NULL;
vector<pProcess> gProcess;

int gProcNum;
int gLinkNum;
int gTileNum = 16;
int g_edge_size= 4;    //How many tiles per edge

int Process::proc_num = 0;
int Tile::cnt = 0;
int Tile::totalTiles = 0;
int Link::cnt = 0;
int MappingNode::cnt = 0;

int main(int argn, char** argv) {

    srand(param.seed);
    gProcNum = 16;
    gTileNum = 16;

    gProcess.clear();

    strcpy(param.exec_str, argv[0]);
    parse_options(argn, argv);

    if (param.parse_error) {
        cerr << ERRO_PARSING_OPTIONS;
        return -1;
    }

    if (param.quit_flag)
        return 0;

    //By calling initialize, classes are constructed, with some randomly
    //generated communication pattern and the default routing table. 
    //The processes parameters can be overloaded by calling parseTGFile()
    //later, and the routing table can be later be override by calling
    //parseRoutingTable()
    if (!initialize()) {
        cerr << ERRO_SYSTEM_INIT;
        return -1;
    }

    mapProcToTile(RANDOM_MAPPING);

    if (param.do_mapping) 
        optimizeMapping();

    if (param.read_mapping) 
        read_mapping_table(param.read_mapping_table_file);

    if (param.dump_mapping) 
        dump_mapping_table(param.dump_mapping_table_file);

    if (param.dump_apcg)
        dump_apcg(param.dump_apcg_file);

    if (param.dump_routing)
        dump_routing_tables(param.routing_table_file);

    analyzeIt();

    //    print_comm_statistics();

    clean();
    return 1;
}



