#!/usr/bin/perl -w
#
# Copyright (c) 2005 Carnegie Mellon University.  All rights
# reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. The end-user documentation included with the redistribution,
#    if any, must include the following acknowledgment:
#       "This product includes software developed by
#        Carnegie Mellon University (http://www.cmu.edu/)."
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
#
# 4. The names "Carnegie Mellon University" and "CMU" must
#    not be used to endorse or promote products derived from this
#    software.
#
# 5. Products derived from this software may not be called
#     "Carnegie Mellon University" or "CMU",
#    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @ProposedRating yellow jingcao@ece.cmu.edu
#
# Description: A simple perl script which is used to remap traces. The old
# trace files are assumed to be with the name of *.trace (e.g. 0.trace --
# 15.trace) and the new trace files will be of the name *.trace.remapped.
########################################################################

use strict ;

my $numArgs = $#ARGV + 1;

if ($numArgs != 1) {
    print ("Usage: \nremap_trace.pl map_file\n");
    exit;
}

my $map_file_name = $ARGV[0];

open(map_file, $map_file_name) ||
    die("Unable to open file " . $map_file_name . " for read.\n");

# map_file is in the format of (for each line)
# old_id     mapped_id
my @new_id = ( 0 );
my $count = 0;

while (my $input_line = <map_file>) {
    if ( $input_line =~ /^\#/ ) {
        next;
    }
    my @id = split(/[ \t]+/, $input_line);
    if ($#id + 1 != 2) {
        die ("Unknown map file format\n");
    }
    my $max = $id[0];
    if ( $max < $id[1] ) {
        $max = $id[1];
    }
    chomp($id[1]);
    if ($max > $#new_id) {
# we need to expand the array
        for ($count = 0; $count <= $max - $#new_id; $count ++) {
            @new_id = (@new_id, -1);
        }
    }
    $new_id[$id[0]] = $id[1];
}

close(map_file);

# now start converting
for ($count = 0; $count <= $#new_id; $count ++) {
    my $old_trace_name = "$count.trace";
    my $new_trace_name = "$new_id[$count].trace.remapped";
    print ("Remapping trace $old_trace_name -> $new_trace_name ... \n");
    open(old_trace, $old_trace_name) ||
        die("Unable to open file " . $old_trace_name . " for read.\n");
    open(new_trace, ">$new_trace_name") ||
        die("Unable to open file " . $new_trace_name . " for write.\n");
    
    # add some header info to the new trace file 
    print new_trace ("# File generated by remapping utility remap_trace.pl.\n");
    print new_trace ("# Remapped as $old_trace_name -> $new_trace_name\n");
    print new_trace ("# Converted content starts below\n");

    while (my $input_line = <old_trace>) {
        if ( $input_line =~ /^\#/ ) {   # comment line should be copied
            print new_trace $input_line;
        }
        elsif ( $input_line =~ /^PERIOD/ ) { # copy period statement
            print new_trace $input_line;
        }
        elsif ( $input_line =~ /^\n/ ) { # duplicate blank lines
            print new_trace $input_line;
        }
        else {   # now this has to be a message record
            my @list = split(/[ \t]+/, $input_line);
            if ($#list + 1 != 3 ) {
                die("Unknown semantics in line " . $input_line);
            }
            else { # remap the destination field
                $list[1] = $new_id[$list[1]];
                print new_trace ($list[0] . "\t" . $list[1] . "\t" . $list[2]);
            }
        }
    }
    close(old_trace);
    close(new_trace);
}


