#!/usr/bin/perl -w
#
# Copyright (c) 2005 Carnegie Mellon University.  All rights
# reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. The end-user documentation included with the redistribution,
#    if any, must include the following acknowledgment:
#       "This product includes software developed by
#        Carnegie Mellon University (http://www.cmu.edu/)."
#    Alternately, this acknowledgment may appear in the software itself,
#    if and wherever such third-party acknowledgments normally appear.
#
# 4. The names "Carnegie Mellon University" and "CMU" must
#    not be used to endorse or promote products derived from this
#    software.
#
# 5. Products derived from this software may not be called
#     "Carnegie Mellon University" or "CMU",
#    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
# ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @ProposedRating yellow jingcao@ece.cmu.edu
#
# Description: util to parse the output log file of nocmmm simulator.
########################################################################

use strict ;

## const definition 
my $verify_mode = 0;
my $latency_mode = 1;

my $warm_up_period = 2000;    # the first 2000 cycles are for warming up

my $mode;

my $num_args = $#ARGV + 1;

if ($num_args != 2) {
    die("Usage: trace_parser.pl [-v|-l] trace_name\n");
}

if ($ARGV[0] eq "-v") {
    $mode = $verify_mode;
}
elsif ($ARGV[0] eq "-l") {
    $mode = $latency_mode;
}
else {
    die("Usage: trace_parser.pl [-v|-l] trace_name\n");
}

my $trace_name = $ARGV[1];

open(trace_file, $trace_name) ||
    die("Unable to open file " . $trace_name . " for read.\n");


my $latency_sum = 0;
my $pkt_num = 0;

if ($mode == $latency_mode) {
    while (my $input_line = <trace_file>) {
        if ($input_line =~ /Latency/) {
            my @list = split(/: /, $input_line);
            if (@list != 3) {
                close(trace_file);
                die("Unknown format\n");
            }
            (my $dummy1, my $cur_time) = split(/ /, $list[0]);
            if ($cur_time >= $warm_up_period) {
                my $last_field = $list[2];
                chomp($last_field);
                ($dummy1, my $dummy2, my $latency) = split(/ /, $last_field);
                $latency_sum = $latency_sum + $latency;
                $pkt_num = $pkt_num + 1;
            }
        }
    }
    print ("Totally received $pkt_num packets after warming up, average packet latency = " . $latency_sum/$pkt_num . "\n");
}


my %hash_pkts;            # hash with pkt size and dst as value, key is (src, pktId)
my $sent_pkts = 0;
my $recv_pkts = 0;

my @list;

my $line_no = 0;
if ($mode == $verify_mode) {
    print ("Perform time consuming verification, please wait patiently ...\n");
    while (my $input_line = <trace_file>) {
        $line_no = $line_no + 1;
        if ($input_line =~ /generated packet/) {  # for source node, add to hash_pkts
            (my $dummy1, my $src, my $dummy2, my $src1, my $dummy3, my $dst) = split(/[()]/, $input_line);
            if (!($src eq $src1)) {
                close(trace_file);
                die("ERROR LINE " . $line_no . " Source address in packet does not match traffic source address\n");
            }
            if ($src eq $dst) {
                close(trace_file);
                die("ERROR LINE " . $line_no . " Traffic source sending to itself\n");
            }
# now let's get the pkt id
            (my $f1, my $f2) = split(/;/, $input_line);
            @list = split(/ /, $f1);
            my $pkt_id = $list[$#list];

            my $key = $src . $pkt_id;

# now let's retrieve the size
            ($f1, $f2) = split(/Size = /, $input_line);
            @list = split(/ /, $f2);
            my $pkt_size = $list[0];
            my $value = $dst . $pkt_size;

            die("ERROR LINE " . $line_no . " Possible duplicated packets\n") if exists $hash_pkts{ $key };

            $hash_pkts{ $key } = $value;

            $sent_pkts = $sent_pkts + 1;
        }
        if ($input_line =~ /Latency/) {  # for sink node, delete from hash_pkts
            @list = split(/: /, $input_line);
            if (@list != 3) {
                die("Unknown format\n");
            }
            (my $dummy1, my $dst, my $dummy2) = split(/[()]/, $list[0]);
            ($dummy1, my $src, $dummy2, my $dst1) = split(/[()]/, $list[1]);
            if (!($dst eq $dst1)) {
                close(trace_file);
                die("ERROR LINE " . $line_no . " Reciever does not match the packet destination address\n");
            }

            (my $f1, my $f2) = split(/;/, $input_line);
            @list = split(/ /, $f1);
            my $pkt_id = $list[$#list];

            ($f1, $f2) = split(/Size = /, $input_line);
            @list = split(/ /, $f2);
            my $pkt_size = $list[0];

            my $key = $src . $pkt_id;
            my $value = $dst . $pkt_size;

            if (!(exists $hash_pkts{ $key })) {
                close(trace_file);
                die("ERROR LINE " . $line_no . " received packet has never been sent\n");
            }
            if (!($hash_pkts{ $key } eq $value)) {
                close(trace_file);
                die("ERROR LINE " . $line_no . " received packet does not match sent packet\n");
            }

            $recv_pkts = $recv_pkts + 1;
        }
    }

    if ($recv_pkts == $sent_pkts) {
        print("Verification passed, totally verified " . $recv_pkts . " packets\n");
    }
    else {
        print("All the received packets match the sent packets, however: \n");
        print("Verification failed, sent packets = " . $sent_pkts . " received packets = " . $recv_pkts . "\n");
        print("This is probably because there are residue packets in the network.\n");
    }
}



close(trace_file);
