/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 */

#ifndef COMMON_HPP_
#define COMMON_HPP_

// Since we are not sure of the copyright info of the orion power library, 
// in this release worm_sim will not support orion power model
// #define USE_ORION   // enable this if to compile with orion power model support 


#include <iostream>
#include <fstream>
#include <cassert>
#include <vector>
#include <utility>

#define trace_it(s) cout<<#s<<" :"<<s<<endl;
#define MAX_LINE 1024

using namespace std;


typedef struct Position {
    unsigned int x;
    unsigned int y;
} Position, *pPosition; 

typedef enum Direction { local = 0, north, south, east, west, extra, invalid_dir } Direction; // UYO: Add extra direction

typedef enum Topology { mesh = 0, torus, invalid_topology } Topology;

typedef enum Routing_scheme { xy = 0, 
                              oe, 
                              oe_fixed, 
                              oe_split,
                              dyad,
			      rtable,   // UYO
                              invalid_scheme
} Routing_scheme;

typedef enum Traffic_mode { uniform = 0,      // uniformly distributed destination
                            transpose1,       // (i,j) -> (N-j, N-i)
                            transpose2,       // (i,j) -> (j->i)
                            hotspot,
                            invalid_distribution 
} Traffic_mode;

bool operator ==(const Position & pos1, const Position & pos2);
bool operator !=(const Position & pos1, const Position & pos2);

ostream & operator <<(ostream & os, const Position & pos);

Direction reverse(Direction dir);


// these are from popnet project. need to decide what to keep
enum mess_type {EVG_, ROUTER_, WIRE_, CREDIT_};
enum routing_type {XY_ , TXY_ };
enum VC_state_type {INIT_, ROUTING_, VC_AB_, SW_AB_, SW_TR_, HOME_};
enum flit_type {HEADER_, BODY_, TAIL_};
enum vc_share_type {SHARE_, MONO_};
enum VC_usage_type {USED_, FREE_};
typedef double time_type;
typedef vector<long> add_type;
typedef pair<long, long> VC_type;
typedef vector<unsigned long long> Data_type;
typedef unsigned long long Atom_type;
const VC_type VC_NULL = VC_type(-1, -1);
#define BUFF_BOUND_ 100
#define WIRE_DELAY_ 0.9
#define PIPE_DELAY_ 1.0
#define CREDIT_DELAY_ 1.0
#define REPORT_PERIOD_ 2000
#define S_ELPS_ 0.00000001
#define ZERO_ 0
#define MAX_32_ static_cast<unsigned long>(0xffffffff)
#define MAX_64_ static_cast<unsigned long long>(0xffffffffffffffff)
#define CORR_EFF_ 0.8
#define POWER_NOM_ 1e9
#define DOUBLE_DELTA 1e-9  // used for double value comparison 


#endif  // COMMON_HPP_
