/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 */

#ifndef NETWORK_HPP_
#define NETWORK_HPP_

#include "common.hpp"
#include "router.hpp"
#include "link.hpp"
#include "traffic.hpp"
#include "channel.hpp"

typedef struct Ch_load {
    class Input_channel * ch;
    double load;
};
typedef Ch_load *pCh_load;

typedef class Network {
private:
    bool success;

    // configuration parameters
    Topology topology;
    unsigned int n_of_rows;
    unsigned int n_of_cols;
    unsigned int n_of_ports;
    unsigned int n_of_extra_links; // UYO
    unsigned int n_of_switch_ports;
    unsigned int n_of_vcs;

    vector<class Router *> routers;
    vector<class Link *> links;
    vector<class Traffic_source *> sources;
    vector<class Traffic_sink *> sinks;

private:
    void calc_channel_load(void);
//     void perf_anal_init_lambdas(double * n_lambda);
//     void perf_anal_init_probs(double * n_prob);
    bool overloaded(double * n_lambda);

public:
    void reset(void); 
    Network(void);
    ~Network(void); 
    class Router * get_router(Position & p);
    class Router * get_router(int index) { return routers[index]; } 
    class Traffic_source * get_traffic_source(Position & p);
    class Traffic_source * get_traffic_source(int index) { return sources[index]; }
    class Traffic_sink * get_traffic_sink(Position & p);
    class Traffic_sink * get_traffic_sink(int index) { return sinks[index]; }
    class Link * connect(class Connector * src,  class Connector * dst, int delay=0);

    Topology get_topology(void) const { return topology; }
    unsigned int get_num_of_rows(void) const { return n_of_rows; }
    unsigned int get_num_of_cols(void) const { return n_of_cols; }
    unsigned int get_num_of_ports(void) const { return n_of_ports; }
    unsigned int get_num_of_vcs(void) const { return n_of_vcs; }

    unsigned int get_num_of_traffic_sources(void) const { return sources.size(); }
    unsigned int get_num_of_traffic_sinks(void) const { return sinks.size(); }

    double get_total_injection_rate(void) const;
    
    void tick(void); 
    bool get_success_flag(void) { return success; }
    //    double analyze_latency(void);
    //     bool dump_perf_anal_lib_equation_file(void);

    bool dump_equation_file(void);
    bool dump_config_file(void);
};

typedef Network *pNetwork;

#endif  // NETWORK_HPP_
