/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 */

#ifndef SWITCH_FABRIC_HPP_
#define SWITCH_FABRIC_HPP_

#include "common.hpp"
#include <vector>

using namespace std;

typedef class Sw_end {
protected:
    class Channel * chan;
    class Sw_end * connected_to;              // which end does it currently connected
public:
    void bind(class Channel * c) { chan = c; }
    void connect(class Sw_end * e) { connected_to = e; }
    void disconnect(void) { connected_to = 0; }
    bool is_busy(void) const { return (connected_to != 0); }
    Sw_end(void) { chan = 0; connected_to = 0; }
    class Sw_end * get_other_end(void) { return connected_to; }
    class Channel * get_bound_channel(void) { return chan; }
} Sw_end, *pSw_end;


typedef class Sw_input_end : public Sw_end {
} Sw_input_end, *pSw_input_end; 


typedef class Sw_output_end : public Sw_end{
private:
} Sw_output_end, *pSw_output_end;


typedef class Switch_fabric {
    double ebit;
    unsigned int size;
    class Router * router;
    vector<class Sw_input_end *> inputs;
    vector<class Sw_output_end *> outputs;

public:
    void bind(class Router * r) { router = r; }
    Switch_fabric(int sz);
    void connect(class Sw_input_end * in, class Sw_output_end * out);
    unsigned int get_size(void) const { return size; }
    class Sw_input_end * get_input_end(int index) { return inputs[index]; }
    class Sw_output_end * get_output_end(int index) { return outputs[index]; }
    void setup_connection(pSw_input_end swi, pSw_output_end swo);
    void teardown_connection(pSw_input_end swi, pSw_output_end swo);
    void set_ebit(double e) { ebit = e; }
    double get_ebit(void) const { return ebit; }
    friend class Arbiter;
} Switch_fabric, *pSwitch_fabric;


#endif  // SWITCH_FABRIC_HPP_
