/*
 * Copyright (c) 2005 Carnegie Mellon University.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        Carnegie Mellon University (http://www.cmu.edu/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Carnegie Mellon University" and "CMU" must
 *    not be used to endorse or promote products derived from this
 *    software.
 *
 * 5. Products derived from this software may not be called
 *     "Carnegie Mellon University" or "CMU",
 *    nor may "Carnegie Mellon University"  or "CMU" appear in their name.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @ProposedRating yellow jingcao@ece.cmu.edu
 */


#include <iostream>
#include <sys/types.h>
#include <cstdlib>
#include <cstdio>
#include <sys/times.h>

#include "version.hpp"
#include "misc.hpp"
#include "channel.hpp"
#include "pkt.hpp"
#include "routing_engine.hpp"
#include "version.hpp"
#include "msg.hpp"
#include "util.hpp"

using namespace std;

Net_clock net_clock;
Param param;

unsigned int Traffic_source::id_base = 0;
unsigned int Traffic_sink::id_base = 0;

int main(int argn, char ** argv) {

    parse_options(argn, argv);

    if (param.parse_error) {
        cerr << ERRO_PARSING_OPTIONS;
        return -1;
    }

    if (param.quit_flag)
        return 0;

    srand48(param.rand_seed);

    Network network;
    if (!network.get_success_flag()) {
        cerr << ERRO_NETWORK_BUILD_ERROR;
        return -1;
    }

    for (vector<char*>::iterator f = param.config_files.begin();
         f < param.config_files.end(); f++) {
        if (apply_config_file(*f)) {
            cerr << ERRO_CONFIG_FILE;
            return -1;
        }
    }
        
    if (post_initialization_configure(network)) {
        cerr << ERRO_NETWORK_BUILD_ERROR;
        return -1;
    }

    //     if (param.analyze_latency) {
    //         cout << "Perform latency analysis ... " << endl;
    //         double latency = network.analyze_latency(); 
    //         if (latency > 0)
    //             cout << "Average latency for a packet: " << latency << endl;
    //         else 
    //             cout << "Average latency for a packet: Inf" << endl;
    //         return 1;
    //     }

    if (param.equation_file) {
        network.dump_equation_file();
        return 1;
    }

    //     if (param.perf_anal_lib_equation_file) {
    //         network.dump_perf_anal_lib_equation_file();
    //         return 1;
    //     }

    if (param.config_file_to_dump) {
        network.dump_config_file();
        return 1;
    }

    clock_t start, end;
    struct tms tmsstart, tmsend;
    start = times(&tmsstart);

    net_clock.reset();
    while (net_clock.tick() < param.simulation_length) {
        network.tick();
    }

    end = times(&tmsend);

    cout << INFO_SIM_DONE;

    print_parameters();
    print_statistics(network);

    if (param.print_simulation_time)
        print_times(end-start, &tmsstart, &tmsend);

    return 1;
}
